
From: Roland McGrath <roland@redhat.com>

It was intended that such things would not be possible because getting into
that code in the first place should be ruled out while exiting.  That
removes the requirement for any special case check in the common path. 
But, it was done too late since it hadn't occurred to me that ->live going
zero itself created a problem.

Signed-off-by: Roland McGrath <roland@redhat.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/kernel/exit.c |   15 ++++++++-------
 1 files changed, 8 insertions(+), 7 deletions(-)

diff -puN kernel/exit.c~panic-in-check_process_timers kernel/exit.c
--- 25/kernel/exit.c~panic-in-check_process_timers	2005-03-07 20:41:52.000000000 -0800
+++ 25-akpm/kernel/exit.c	2005-03-07 21:14:15.000000000 -0800
@@ -753,15 +753,8 @@ static void exit_notify(struct task_stru
 		state = EXIT_DEAD;
 	tsk->exit_state = state;
 
-	/*
-	 * Clear these here so that update_process_times() won't try to deliver
-	 * itimer, profile or rlimit signals to this task while it is in late exit.
-	 */
 	tsk->it_virt_value = cputime_zero;
 	tsk->it_prof_value = cputime_zero;
- 	tsk->it_virt_expires = cputime_zero;
- 	tsk->it_prof_expires = cputime_zero;
-	tsk->it_sched_expires = 0;
 
 	write_unlock_irq(&tasklist_lock);
 
@@ -804,6 +797,14 @@ fastcall NORET_TYPE void do_exit(long co
 	tsk->flags |= PF_EXITING;
 	del_timer_sync(&tsk->real_timer);
 
+	/*
+	 * Make sure we don't try to process any timer firings
+	 * while we are already exiting.
+	 */
+ 	tsk->it_virt_expires = cputime_zero;
+ 	tsk->it_prof_expires = cputime_zero;
+	tsk->it_sched_expires = 0;
+
 	if (unlikely(in_atomic()))
 		printk(KERN_INFO "note: %s[%d] exited with preempt_count %d\n",
 				current->comm, current->pid,
_
